#!/usr/bin/bash
#
# Copyright (c) 2024 Alibaba Group Holding Limited
# vtoa is licensed under Mulan PSL v2.
# You can use this software according to the terms and conditions of the Mulan PSL v2.
# You may obtain a copy of Mulan PSL v2 at:
#          http://license.coscl.org.cn/MulanPSL2
# THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
# EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
# MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
# See the Mulan PSL v2 for more details.
#

# used in .spec-%post/%preun, start vtoa by kernel version
# < 5.10-015: do not install
# >= 5.10-017: vtoa-bpf
# >= 5.10-015 && < 5.10-017: vtoa.ko

ko_conf=/etc/modules-load.d/vtoa.conf
ko_name=vtoa

bpf_remove() {
	echo "vtoa-bpf remove..."
	systemctl stop vtoa
	systemctl disable vtoa
	echo "vtoa-bpf removed."
}

ko_remove() {
	echo "vtoa.ko remove..."
	modinfo $ko_name
	if [[ $? -ne 0 ]]
	then
		echo "Warning: vtoa.ko not found, turns to vtoa-bpf."
		bpf_remove
		return 0
	fi

	modprobe -r $ko_name
	# if FILE not exists
	if [ ! -f "$ko_conf" ]
	then
		# the config file is not found, this could be caused by:
		# (1) deleted by user manually;
		# (2) the kernel version is too old to be supported.
		# So we exit directly without errors.
		echo "Warning: File '$ko_conf' not found"
		return 0
	fi

	# remove ko_name in FILE
	echo "Removing '$ko_name' from '$ko_conf'..."
	sed -i "/$ko_name/d" "$ko_conf"

	# if FILE is empty, remove it
	if [ ! -s "$ko_conf" ]
	then
		echo "File '$ko_conf' is empty. Deleting file..."
		rm "$ko_conf"
	fi
	echo "vtoa.ko removed."
}

bpf_launch() {
	echo "vtoa-bpf launch..."
	systemctl enable vtoa
	systemctl start vtoa
	echo "vtoa-bpf is ready."
}

ko_launch() {
	echo "vtoa.ko launch..."

	modinfo $ko_name
	if [[ $? -ne 0 ]]
	then
		echo "Warning: vtoa.ko not found, turns to vtoa-bpf."
		bpf_launch
		return 0
	fi

	# load vtoa.ko
	if lsmod | grep -q "^${ko_name}\b"
	then
		echo "vtoa.ko is already loaded."
	else
		echo "Loading vtoa.ko"
		modprobe $ko_name
	fi

	# vtoa.ko is usable, setup auto load
	if [ -f "$ko_conf" ]
	then
		echo "File ${ko_conf} exists."
		# is "vtoa" already in the file?
		if grep -q -x "$ko_name" "$ko_conf"; then
			echo "vtoa already exists in ${ko_conf}."
		else
			echo "add vtoa as a new line to ${ko_conf}."
			echo vtoa >> ${ko_conf}
		fi
	else
		echo "File ${ko_conf} does not exist, creating it."
		echo vtoa > ${ko_conf}
	fi
	echo "vtoa.ko is ready."
}

# 1 means launch, else remove
mng_cmd=$1

# always do both remove if mng_cmd is 0, because ko and bpf can be launched together by user
ko_manage() {
	if [[ "$mng_cmd" == "1" ]]
	then
		ko_launch
	else
		bpf_remove
		ko_remove
	fi
}

bpf_manage() {
	if [[ "$mng_cmd" == "1" ]]
	then
		bpf_launch
	else
		bpf_remove
		ko_remove
	fi
}

# version is compared with `sort -V`, e.g., following versions are all larger than $bpf_kernel_version:
# 5.10.134-017, 5.10.135-*, 5.11.*, 6.*
bpf_kernel_version="5.10.134-017"
ko_kernel_version="5.10.134-015"

cur_kernel_ver=$(uname -r)
if [[ "$bpf_kernel_version" == "$(echo -e "$bpf_kernel_version\n$cur_kernel_ver" | sort -V | head -n1)" ]]
then
	# >= 5.10-017
	echo "kernel version >= $bpf_kernel_version: vtoa-bpf."
	bpf_manage

elif [[ "$ko_kernel_version" == "$(echo -e "$ko_kernel_version\n$cur_kernel_ver" | sort -V | head -n1)" ]]
then
	# >= 5.10-015
	echo "kernel version $ko_kernel_version ~ $bpf_kernel_version: vtoa.ko."
	ko_manage

else
	# < 5.10-015
	echo "kernel version < $ko_kernel_version: not support, do nothing."
fi